import { createReducer, on } from '@ngrx/store';
import {
  initialState,
  adapterBuddy2,
  adapterGroup2,
  adapterRoom,
  adapterRoomUser,
  adapterRoomUserShort
} from './state';
import {
  buddy2Success,
  group2Success,
  roomSuccess,
  updateRoomForNewEventMessage,
  refreshRoomSuccess,
  updateUnreadCount,
  createGroupSuccess,
  addBuddySuccess,
  delBuddySuccess
} from './actions';
import {
  RoomUserDetailData,
  RoomUserData,
  GroupDetailData
} from '@ucap-webmessenger/protocol-sync';

import * as AuthenticationStore from '@app/store/account/authentication';
import * as RoomStore from '@app/store/messenger/room';
import { RoomInfo } from '@ucap-webmessenger/protocol-room';

export const reducer = createReducer(
  initialState,
  on(buddy2Success, (state, action) => {
    return {
      ...state,
      buddy2: adapterBuddy2.upsertMany(action.buddyList, {
        ...state.buddy2,
        syncDate: action.syncDate
      })
    };
  }),

  on(group2Success, (state, action) => {
    return {
      ...state,
      group2: adapterGroup2.upsertMany(action.groupList, {
        ...state.group2,
        syncDate: action.syncDate
      })
    };
  }),

  on(roomSuccess, (state, action) => {
    const roomUserList: RoomUserDetailData[] = [];
    const roomUserShortList: RoomUserData[] = [];

    for (const key in action.roomUserInfoMap) {
      if (action.roomUserInfoMap.hasOwnProperty(key)) {
        const element = action.roomUserInfoMap[key];
        if (!!element.userInfoList && 0 < element.userInfoList.length) {
          roomUserList.push({ roomSeq: key, userInfos: element.userInfoList });
        }
        if (
          !!element.userInfoShortList &&
          0 < element.userInfoShortList.length
        ) {
          roomUserShortList.push({
            roomSeq: key,
            userInfos: element.userInfoShortList
          });
        }
      }
    }

    let unReadCount = 0;
    /** SYNC ROOM 으로 정보 수집시 증분값을 받는다면 기존 state 의 정보를 뒤져 noReadCount 를 계산 후 중분 값에 대한 noReadCount 를 더한다. */
    // tslint:disable-next-line: forin
    for (const key in state.room.entities) {
      const value = state.room.entities[key];
      unReadCount += value.noReadCnt;
    }
    action.roomList.map(item => (unReadCount += item.noReadCnt));

    return {
      ...state,
      room: adapterRoom.upsertMany(action.roomList, {
        ...state.room,
        syncDate: action.syncDate
      }),
      roomUser: adapterRoomUser.upsertMany(roomUserList, {
        ...state.roomUser
      }),
      roomUserShort: adapterRoomUserShort.upsertMany(roomUserShortList, {
        ...state.roomUserShort
      }),
      chatUnreadCount: unReadCount
    };
  }),

  on(updateRoomForNewEventMessage, (state, action) => {
    const roomInfo = {
      ...state.room.entities[action.roomSeq],
      finalEventDate: action.info.sendDate,
      finalEventMessage: action.info.sentMessage
    };

    return {
      ...state,
      room: adapterRoom.updateOne(
        { id: action.roomSeq, changes: roomInfo },
        { ...state.room }
      )
    };
  }),

  on(RoomStore.updateSuccess, (state, action) => {
    const roomInfo: RoomInfo = {
      ...state.room.entities[action.res.roomSeq],
      roomName: action.res.roomName,
      receiveAlarm: action.res.receiveAlarm
    };

    return {
      ...state,
      room: adapterRoom.updateOne(
        { id: action.res.roomSeq, changes: roomInfo },
        { ...state.room }
      )
    };
  }),

  on(refreshRoomSuccess, (state, action) => {
    const roomUserList: RoomUserDetailData[] = [];
    const roomUserShortList: RoomUserData[] = [];

    if (action.userInfoList) {
      roomUserList.push({
        roomSeq: action.roomInfo.roomSeq,
        userInfos: action.userInfoList
      });
    }
    if (action.userInfoShortList) {
      roomUserShortList.push({
        roomSeq: action.roomInfo.roomSeq,
        userInfos: action.userInfoShortList
      });
    }

    return {
      ...state,
      room: adapterRoom.upsertOne(action.roomInfo, {
        ...state.room
      }),
      roomUser: adapterRoomUser.upsertMany(roomUserList, {
        ...state.roomUser
      }),
      roomUserShort: adapterRoomUserShort.upsertMany(roomUserShortList, {
        ...state.roomUserShort
      })
    };
  }),

  on(updateUnreadCount, (state, action) => {
    const roomInfo: RoomInfo = {
      ...state.room.entities[action.roomSeq],
      noReadCnt: action.noReadCnt
    };

    let unReadCount = 0;
    if (action.noReadCnt === 0) {
      // tslint:disable-next-line: forin
      for (const key in state.room.entities) {
        if (key !== action.roomSeq) {
          const value = state.room.entities[key];
          unReadCount += value.noReadCnt;
        }
      }
    } else {
      unReadCount = state.chatUnreadCount + 1;
    }

    return {
      ...state,
      room: adapterRoom.updateOne(
        { id: action.roomSeq, changes: roomInfo },
        { ...state.room }
      ),
      chatUnreadCount: unReadCount
    };
  }),

  /** 새 그룹 추가. */
  on(createGroupSuccess, (state, action) => {
    const groupInfo: GroupDetailData = {
      seq: action.groupSeq,
      name: action.groupName,
      isActive: true,
      userSeqs: []
    };
    return {
      ...state,
      group2: adapterGroup2.addOne(groupInfo, {
        ...state.group2
      })
    };
  }),

  on(delBuddySuccess, (state, action) => {
    return {
      ...state,
      buddy2: adapterBuddy2.removeMany(action.userSeqs, {
        ...state.buddy2
      })
    };
  })
);
